//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Job/ParameterBackup.cpp
//! @brief     Implements class ParameterBackup.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2023
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Model/Job/ParameterBackup.h"
#include "GUI/Support/XML/UtilXML.h"

namespace {
namespace Tag {

const QString Title("Title");
const QString BackupValues("BackupValues");
const QString BackupValue("BackupValue");

} // namespace Tag
} // namespace

ParameterBackup::ParameterBackup(const QString& title)
    : m_title(title)
{
}

double ParameterBackup::value(const QString& link)
{
    return m_backupValues[link];
}

void ParameterBackup::setValue(const QString& link, double d)
{
    m_backupValues[link] = d;
}

bool ParameterBackup::contains(const QString& link)
{
    return m_backupValues.contains(link);
}

void ParameterBackup::writeTo(QXmlStreamWriter* w) const
{
    XML::writeAttribute(w, XML::Attrib::version, uint(1));

    // title
    w->writeStartElement(Tag::Title);
    XML::writeAttribute(w, XML::Attrib::value, m_title);
    w->writeEndElement();

    // values
    w->writeStartElement(Tag::BackupValues);
    for (auto v = m_backupValues.cbegin(); v != m_backupValues.cend(); v++) {
        w->writeEmptyElement(Tag::BackupValue);
        XML::writeAttribute(w, XML::Attrib::id, v.key());
        XML::writeAttribute(w, XML::Attrib::value, v.value());
    }
    w->writeEndElement();
}

void ParameterBackup::readFrom(QXmlStreamReader* r)
{
    m_backupValues.clear();

    const uint version = XML::readUIntAttribute(r, XML::Attrib::version);
    Q_UNUSED(version)

    while (r->readNextStartElement()) {
        QString tag = r->name().toString();

        // title
        if (tag == Tag::Title) {
            XML::readAttribute(r, XML::Attrib::value, &m_title);
            XML::gotoEndElementOfTag(r, tag);

            // values
        } else if (tag == Tag::BackupValues) {
            readValues(r);
            XML::gotoEndElementOfTag(r, tag);

        } else
            r->skipCurrentElement();
    }
}

void ParameterBackup::readValues(QXmlStreamReader* r)
{
    while (r->readNextStartElement()) {
        QString valTag = r->name().toString();

        if (valTag == Tag::BackupValue) {
            QString link;
            double d;
            XML::readAttribute(r, XML::Attrib::id, &link);
            XML::readAttribute(r, XML::Attrib::value, &d);
            m_backupValues[link] = d;
            XML::gotoEndElementOfTag(r, valTag);

        } else
            r->skipCurrentElement();
    }
}
