//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Mask/MaskContainerItem.h
//! @brief     Defines class MaskContainerItem.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_MASK_MASKCONTAINERITEM_H
#define BORNAGAIN_GUI_MODEL_MASK_MASKCONTAINERITEM_H

#include "GUI/Model/Descriptor/SelectionProperty.h"
#include "GUI/Model/Mask/MaskItemCatalog.h"
#include "GUI/Model/Mask/MaskRoot.h"
#include <QModelIndex>

class MaskContainerModel;
class MaskItem;
class MessageService;
class RegionOfInterestItem;

//! Container holding various masks as children

class MaskContainerItem : public MaskRoot {
public:
    MaskContainerItem();
    ~MaskContainerItem();

    QVector<MaskItem*> maskItems() const;

    //! Insert mask at given row.
    virtual void insertMask(int row, MaskItem* maskItem);

    virtual void addMaskItem(MaskItem* maskItem);

    //! Move mask to a given row
    virtual void moveMask(int from_row, int to_row);

    virtual void removeMaskAt(int row);

    //! Can be nullptr.
    RegionOfInterestItem* regionOfInterestItem() const;

    //! Remove all masks
    void clear();

    //! Return true if the container has no MaskItems
    bool isEmpty() const;

    //! Return the number of MaskItems
    int size() const;

    //! Return the MaskItem at a given index
    int indexOfItem(const MaskItem* maskItem) const;

    //! Return the index corresponding to a given MaskItem
    MaskItem* at(const int idx);

    //! Serialize/deserialize
    void writeTo(QXmlStreamWriter* w) const;
    void readFrom(QXmlStreamReader* r, MessageService* messageService = nullptr);

    //! Update numbers in mask names
    void updateMaskNames();

    const QModelIndex rootIndex;

protected:
    SelectionVector<MaskItemCatalog> m_maskItems;
};

#endif // BORNAGAIN_GUI_MODEL_MASK_MASKCONTAINERITEM_H
