//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Canvas/ProgressCanvas.cpp
//! @brief     Implement class ProgressCanvas
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Canvas/ProgressCanvas.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Job/FitSuiteItem.h"
#include "GUI/Model/Job/JobItem.h"
#include "GUI/View/Plotter/HistogramPlot.h"
#include <QVBoxLayout>

ProgressCanvas::ProgressCanvas()
    : m_histPlot(new HistogramPlot)
{
    setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);

    auto* layout = new QVBoxLayout(this);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->addWidget(m_histPlot);

    setStyleSheet("background-color:white;");
}

void ProgressCanvas::setJobItem(JobItem* item)
{
    ASSERT(item);
    m_job_item = item;

    connect(item->fitSuiteItem(), &FitSuiteItem::iterationCountChanged, this,
            &ProgressCanvas::onIterationCountChanged, Qt::UniqueConnection);
}

void ProgressCanvas::onIterationCountChanged(int iter)
{
    double chi = m_job_item->fitSuiteItem()->chi2();
    if (iter == 1)
        m_histPlot->clearData();
    m_histPlot->addData(static_cast<double>(iter), chi);
}
