//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Canvas/ProjectionsWidget.h
//! @brief     Defines class ProjectionsWidget.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_CANVAS_PROJECTIONSWIDGET_H
#define BORNAGAIN_GUI_VIEW_CANVAS_PROJECTIONSWIDGET_H

#include "GUI/View/Scene/MaskEditorFlags.h"
#include <QTabWidget>

class Data2DItem;
class ProjectionsPlot;

//! Holds tabs of vertical and horizontal projections, located at the bottom of ProjectionsEditor.

class ProjectionsWidget : public QWidget {
    Q_OBJECT
public:
    ProjectionsWidget(QWidget* parent = nullptr);

    void setData2DItem(Data2DItem* data2DItem);
    void disconnectItem();

signals:
    void changeActivityRequest(MaskEditorFlags::Activity value);

public slots:
    void onActivityChanged(MaskEditorFlags::Activity value);
    void onMarginsChanged(double left, double right);

private slots:
    void onTabChanged(int tab_index);

private:
    void setConnected(bool isConnected);

    ProjectionsPlot* m_xProjection;
    ProjectionsPlot* m_yProjection;
    QTabWidget* m_tabWidget;
};

#endif // BORNAGAIN_GUI_VIEW_CANVAS_PROJECTIONSWIDGET_H
