//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Data/DatafileEditor.h
//! @brief     Defines class DatafileEditor.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_DATA_DATAFILEEDITOR_H
#define BORNAGAIN_GUI_VIEW_DATA_DATAFILEEDITOR_H

#include <QComboBox>
#include <QWidget>

class DatafileItem;
class ProjectDocument;

//! For viewing and changing properties of a loaded datafile.
//!
//! Shown at the bottom of DatafilesSelector.
//!
//! Currently provides one single button, for linking the file to an instrument model.

class DatafileEditor : public QWidget {
    Q_OBJECT
public:
    DatafileEditor(QWidget* parent, ProjectDocument* document);

    QSize sizeHint() const override { return QSize(64, 135); }
    QSize minimumSizeHint() const override { return QSize(64, 128); }

    //! Set current DatafileItem to display in instrument selector.
    void setDatafileItem(DatafileItem* realItem);

signals:
    void instrumentUpdated(DatafileItem* realItem);

private:
    //! Processes user interaction with instrument selector combo. If there is realItem,
    //! it will be linked with selected instrument.
    void onInstrumentComboIndexChanged(int index);

    void updateInstrumentComboEntries();
    void updateInstrumentComboIndex();

    QComboBox* m_instrumentCombo;
    DatafileItem* m_currentDatafileItem;
    ProjectDocument* m_document;
};

#endif // BORNAGAIN_GUI_VIEW_DATA_DATAFILEEDITOR_H
