//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Plotter/ColorMap.h
//! @brief     Defines class ColorMap.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_PLOTTER_COLORMAP_H
#define BORNAGAIN_GUI_VIEW_PLOTTER_COLORMAP_H

#include "GUI/View/Plotter/ScientificPlot.h"

class Data2DItem;
class QCPColorMap;
class QCPColorScale;
class QCPLayoutGrid;
class QCPRange;
class UpdateTimer;

//! Presents 2D intensity data from Data2DItem as color map.

//! Provides a minimal functionality for data plotting and axes interaction.
//! Is used as a component of more complicated plotting widgets.

class ColorMap : public ScientificPlot {
    Q_OBJECT
public:
    explicit ColorMap();

    void itemToMap(Data2DItem* item);

    QSize sizeHint() const override { return QSize(500, 400); }
    QSize minimumSizeHint() const override { return QSize(128, 128); }

    QCPColorScale* colorScale() { return m_colorScale; }

    //! Returns rectangle representing current axes zoom state in widget coordinates
    QRectF viewportRectangleInWidgetCoordinates();

    QString infoString(const QPointF& pos) const override;

signals:
    void marginsChanged(double left, double right);

private:
    Data2DItem* data2DItem() const { return m_data_item; }

    void onIntensityModified();
    void onUnitsChanged();
    void setGradient();
    void setInterpolation();
    void setLogz();

    void onDataRangeChanged(QCPRange range);
    void onXaxisRangeChanged(QCPRange range);
    void onYaxisRangeChanged(QCPRange range);
    void replot();
    void onTimeToReplot();
    void marginsChangedNotify();

    void setAxesRangeConnected(bool isConnected);
    void setDataRangeConnected(bool isConnected);
    void setUpdateTimerConnected(bool isConnected);

    void setFixedColorMapMargins();

    void setAxesRangeFromItem();
    void setAxesZoomFromItem();
    void setAxesLabelsFromItem();
    void setDataFromItem();
    void setColorScaleAppearanceFromItem();
    void setDataRangeFromItem();
    void setColorScaleVisible();

    Data2DItem* m_data_item = nullptr;
    QCPColorMap* m_qcpMap;
    QCPColorScale* m_colorScale;
    QCPLayoutGrid* m_colorBarLayout;
    UpdateTimer* m_updateTimer;
};

#endif // BORNAGAIN_GUI_VIEW_PLOTTER_COLORMAP_H
