//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/ProShape/SceneAdaptor.cpp
//! @brief     Implements class SceneAdaptor.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/ProShape/SceneAdaptor.h"
#include "GUI/View/Plotter/ColorMap.h"
#include <QEvent>
#include <qcustomplot.h>

SceneAdaptor::SceneAdaptor()
    : m_plot(nullptr)
{
}

qreal SceneAdaptor::toSceneX(qreal v) const
{
    return m_plot ? m_plot->customPlot()->xAxis->coordToPixel(v) : v;
}

qreal SceneAdaptor::toSceneY(qreal v) const
{
    return m_plot ? m_plot->customPlot()->yAxis->coordToPixel(v) : v;
}

qreal SceneAdaptor::fromSceneX(qreal p) const
{
    return m_plot ? m_plot->customPlot()->xAxis->pixelToCoord(p) : p;
}

qreal SceneAdaptor::fromSceneY(qreal p) const
{
    return m_plot ? m_plot->customPlot()->yAxis->pixelToCoord(p) : p;
}

void SceneAdaptor::setColorMapPlot(ColorMap* plot)
{
    m_plot = plot;
    if (m_plot)
        m_plot->installEventFilter(this);
}

bool SceneAdaptor::eventFilter(QObject* object, QEvent* event)
{
    Q_UNUSED(object);
    if (event->type() == QEvent::Resize || event->type() == QEvent::UpdateRequest) {
        m_viewport_rectangle = m_plot->viewportRectangleInWidgetCoordinates();
        emit update_request();
        return false; // allow further handling
    }
    return true; // stop further handling
}
