//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Project/ProjectsView.cpp
//! @brief     Implements class ProjectsView.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Project/ProjectsView.h"
#include "GUI/Support/Util/Path.h"
#include "GUI/View/Manager/ProjectManager.h"
#include "GUI/View/Tool/LayoutUtil.h"
#include "GUI/View/Widget/GroupBoxes.h"
#include "GUI/View/Widget/StyledToolbar.h"
#include <QCommandLinkButton>
#include <QDesktopServices>
#include <QDir>
#include <QUrl>
#include <QVBoxLayout>

ProjectsView::ProjectsView(QWidget* parent)
    : QWidget(parent)
{
    ProjectManager* pm = ProjectManager::instance();

    auto* layout = new QVBoxLayout;
    setLayout(layout);

    auto* toolbar = new StyledToolbar(this);
    layout->addWidget(toolbar);
    toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

    auto* actionNew = new QAction;
    toolbar->addAction(actionNew);
    actionNew->setText("Create new project");
    actionNew->setIcon(QIcon(":/images/shape-square-plus.svg"));
    actionNew->setIconText("New");
    actionNew->setToolTip("Create new project");
    connect(actionNew, &QAction::triggered, [=]() { pm->newProject(); });

    auto* actionLoad = new QAction(this);
    toolbar->addAction(actionLoad);
    actionLoad->setText("Load project from file");
    actionLoad->setIcon(QIcon(":/images/import.svg"));
    actionLoad->setIconText("Load");
    actionLoad->setToolTip("Load project from file");
    connect(actionLoad, &QAction::triggered, [=]() { pm->openProject(); });

    auto* prjList = new QVBoxLayout;
    layout->addLayout(prjList);

    auto updateProjects = [prjList, pm]() {
        GUI::Util::Layout::clearLayout(prjList);
        for (const auto& file : pm->recentProjects()) {
            auto* button = new QCommandLinkButton;
            button->setText(QFileInfo(file).baseName());
            button->setDescription(GUI::Path::withTildeHomePath(QDir::toNativeSeparators(file)));
            button->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
            connect(button, &QCommandLinkButton::clicked, [pm, file] { pm->openProject(file); });

            prjList->addWidget(button);
        }
        // m_ui->recentProjectsLayout->addStretch(1);
    };
    connect(pm, &ProjectManager::recentListModified, [=]() { updateProjects(); });
    updateProjects();

    layout->addStretch(1);
}
