//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Scene/MaskGraphicsView.h
//! @brief     Defines class MaskGraphicsView.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SCENE_MASKGRAPHICSVIEW_H
#define BORNAGAIN_GUI_VIEW_SCENE_MASKGRAPHICSVIEW_H

#include "GUI/View/Scene/MaskEditorFlags.h"
#include <QGraphicsView>
#include <QWheelEvent>

class MaskGraphicsScene;

//! Graphics view for MaskEditorCanvas

class MaskGraphicsView : public QGraphicsView {
    Q_OBJECT
public:
    MaskGraphicsView(MaskGraphicsScene* scene, QWidget* parent = nullptr);
    QSize sizeHint() const override { return QSize(512, 512); }
    QSize minimumSizeHint() const override { return QSize(128, 128); }

public slots:
    void onResetViewRequest();

signals:
    void changeActivityRequest(MaskEditorFlags::Activity);
    void deleteSelectedRequest();

private:
    void wheelEvent(QWheelEvent* event) override;
    void resizeEvent(QResizeEvent* event) override;
    void keyPressEvent(QKeyEvent* event) override;
    void keyReleaseEvent(QKeyEvent* event) override;

    bool isControlButtonIsPressed(QWheelEvent* event);
    void cancelCurrentDrawing();

    void setZoomValue(double zoom_value);
    void decreaseZoomValue();
    void increaseZoomValue();

    MaskGraphicsScene* m_scene;
    double m_current_zoom_value;
};

#endif // BORNAGAIN_GUI_VIEW_SCENE_MASKGRAPHICSVIEW_H
