//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Scene/RegionOfInterestDisplay.cpp
//! @brief     Defines class RegionOfInterestDisplay.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Scene/RegionOfInterestDisplay.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/View/ProShape/MaskUtil.h"
#include "GUI/View/ProShape/SceneAdaptor.h"
#include <QPainter>

RegionOfInterestDisplay::RegionOfInterestDisplay(RegionOfInterestItem* item)
    : RectangleDisplay(item)
{
}

QRectF RegionOfInterestDisplay::boundingRect() const
{
    return mapFromScene(m_adaptor->viewportRectangle()).boundingRect();
}

void RegionOfInterestDisplay::update_view()
{
    RectangleDisplay::update_view();
    m_bounding_rect = m_adaptor->viewportRectangle();
    update();
}

void RegionOfInterestDisplay::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
{
    QPainterPath outerRect;
    outerRect.addRect(boundingRect());

    QPainterPath innerRect;
    innerRect.addRect(m_mask_rect);

    painter->setBrush(MaskUtil::getMaskBrush(true));
    painter->drawPath(outerRect.subtracted(innerRect));
}
