//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Setup/MaskEditorToolpanel.h
//! @brief     Defines class MaskEditorToolpanel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SETUP_MASKEDITORTOOLPANEL_H
#define BORNAGAIN_GUI_VIEW_SETUP_MASKEDITORTOOLPANEL_H

#include <QFormLayout>
#include <QItemSelection>
#include <QItemSelectionModel>
#include <QListView>
#include <QModelIndex>
#include <QWidget>
#include <functional>

class Data2DItem;
class MaskContainerModel;
class MaskItem;
class RealLimits;

//! Tool widget for MaskEditor

class MaskEditorToolpanel : public QWidget {
    Q_OBJECT
public:
    MaskEditorToolpanel();

    QSize sizeHint() const override;
    QSize minimumSizeHint() const override;

    void setContextMPP(MaskContainerModel* containerModel);

    void resetContextMPP();
    void setPanelHidden(bool hidden);
    QItemSelectionModel* selectionModel();

signals:
    void itemContextMenuRequest(const QPoint& point);

private slots:
    void onSelectionChanged(const QItemSelection& selected, const QItemSelection&);
    void onCustomContextMenuRequested(const QPoint& point);

private:
    //! Set the current mask and creates the UI to edit the mask's properties
    void setCurrentMaskItem(MaskItem* maskItem);

    //! Add a spinbox to edit a mask's double value
    void addMaskSpinBox(const QString& label, std::function<double()> getter,
                        std::function<void(double)> setter, const RealLimits& limits);

    //! Add a checkbox to edit a mask's boolean value
    void addMaskCheckBox(const QString& title, std::function<bool()> getter,
                         std::function<void(bool)> setter);

    //! Returns the mask item for the given index, or nullptr if invalid.
    MaskItem* maskItemForIndex(const QModelIndex& index);

    //! Creates the UI to edit the current mask's properties
    void createMaskEditorUI();

    QListView* m_listView;
    MaskContainerModel* m_maskContainerModel;
    QFormLayout* m_maskPropertiesLayout;
    MaskItem* m_currentMaskItem; //!< the mask item whose properties shall be edited
    bool m_inhibitSelectionChange = false;
};

#endif // BORNAGAIN_GUI_VIEW_SETUP_MASKEDITORTOOLPANEL_H
