//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Shape/IShapeDisplay.h
//! @brief     Defines interface IShapeDisplay.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SHAPE_ISHAPEDISPLAY_H
#define BORNAGAIN_GUI_VIEW_SHAPE_ISHAPEDISPLAY_H

#include "GUI/View/ProShape/MaskType.h"
#include <QGraphicsObject>
#include <QPainter>

class MaskRoot;
class SceneAdaptor;

//! Main interface class for views representing MaskItems, Projections on graphics scene.

class IShapeDisplay : public QGraphicsObject {
    Q_OBJECT
public:
    int type() const override { return MaskType::IMASKVIEW; }

    QRectF boundingRect() const override;

    virtual MaskRoot* parameterizedItem() const = 0;

    void setSceneAdaptor(const SceneAdaptor* adaptor);

    virtual void addView(IShapeDisplay* childView);

protected slots:
    //! update visual appearance of view (triggered by SceneAdaptor)
    virtual void update_view() = 0;
    virtual void onChangedX() {}
    virtual void onChangedY() {}
    virtual void onPropertyChange() {}

protected:
    explicit IShapeDisplay(MaskRoot* item);

    void paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*) override;

    qreal toSceneX(qreal value) const;
    qreal toSceneY(qreal value) const;

    //! convert scene coordinates to ColorMap plot coordinates
    qreal fromSceneX(qreal value) const;
    qreal fromSceneY(qreal value) const;
    void setBlockOnProperty(bool value);

    const SceneAdaptor* m_adaptor;
    QRectF m_bounding_rect;

private:
    void onGeometryChange();
    void onVisibilityChange();
    //! Returns the shape with all masking already applied.
    QPainterPath maskedShape() const;

    bool m_block_on_property_change;
};

#endif // BORNAGAIN_GUI_VIEW_SHAPE_ISHAPEDISPLAY_H
