//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Shape/LineDisplays.h
//! @brief     Defines classes VerticalLineDisplay and HorizontalLineDisplay.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SHAPE_LINEDISPLAYS_H
#define BORNAGAIN_GUI_VIEW_SHAPE_LINEDISPLAYS_H

#include "GUI/View/Shape/IShapeDisplay.h"

class HorizontalLineItem;
class VerticalLineItem;

//! This is a view of VerticalLineItem mask

class VerticalLineDisplay : public IShapeDisplay {
    Q_OBJECT
public:
    int type() const override { return MaskType::VERTICALLINE; }

    explicit VerticalLineDisplay(VerticalLineItem* item);

    QPainterPath shape() const override;
    MaskRoot* parameterizedItem() const override;

protected slots:
    void update_view() override;
    void onChangedX() override;
    void onPropertyChange() override;

protected:
    void paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*) override;
    QVariant itemChange(GraphicsItemChange change, const QVariant& value) override;

private:
    VerticalLineItem* m_item;
};


//! This is a view of HorizontalLineItem mask

class HorizontalLineDisplay : public IShapeDisplay {
    Q_OBJECT
public:
    int type() const override { return MaskType::HORIZONTALLINE; }

    explicit HorizontalLineDisplay(HorizontalLineItem* item);

    QPainterPath shape() const override;
    MaskRoot* parameterizedItem() const override;

protected slots:
    void update_view() override;
    void onChangedY() override;
    void onPropertyChange() override;

protected:
    void paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*) override;
    QVariant itemChange(GraphicsItemChange change, const QVariant& value) override;

private:
    HorizontalLineItem* m_item;
};

#endif // BORNAGAIN_GUI_VIEW_SHAPE_LINEDISPLAYS_H
