//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Shape/MaskContainerDisplay.h
//! @brief     Defines class MaskContainerDisplay.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SHAPE_MASKCONTAINERDISPLAY_H
#define BORNAGAIN_GUI_VIEW_SHAPE_MASKCONTAINERDISPLAY_H

#include "GUI/View/ProShape/SizeHandleElement.h"
#include "GUI/View/Shape/IShapeDisplay.h"
#include <QMap>

class MaskContainerItem;

//! The MaskContainerDisplay is nothing more than just transparent rectangle to cover axes area
//! of ColorMapPlot inside MaskGraphicsScene. The goal of this rectangle is to hide all MaskViews
//! if they are going outside of ColorMapPlot.
//!
//! The size of the rectangle always matches axes viewport at any zoom level.
//! All MasksViews are added to MaskContainerDisplay as children.

class MaskContainerDisplay : public IShapeDisplay {
    Q_OBJECT
public:
    int type() const override { return MaskType::MASKCONTAINER; }

    MaskContainerDisplay(MaskContainerItem* item);

    MaskRoot* parameterizedItem() const override;

private slots:
    void update_view() override;

private:
    void paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*) override;

    MaskContainerItem* m_item;
};

#endif // BORNAGAIN_GUI_VIEW_SHAPE_MASKCONTAINERDISPLAY_H
