//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Shape/PolygonDisplay.h
//! @brief     Defines class PolygonDisplay.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SHAPE_POLYGONDISPLAY_H
#define BORNAGAIN_GUI_VIEW_SHAPE_POLYGONDISPLAY_H

#include "GUI/View/Shape/IShapeDisplay.h"
#include <QPolygonF>

class PolygonItem;

//! This is a View of polygon mask (represented by PolygonItem) on GraphicsScene.

class PolygonDisplay : public IShapeDisplay {
    Q_OBJECT
public:
    int type() const override { return MaskType::POLYGON; }

    explicit PolygonDisplay(PolygonItem* item);

    MaskRoot* parameterizedItem() const override;

    void addView(IShapeDisplay* childView) override;
    bool isClosedPolygon();
    QPointF lastAddedPoint() const;
    QPainterPath shape() const override;

public slots:
    bool closePolygonIfNecessary();
    void onClosePolygonRequest(bool value);

protected slots:
    void update_view() override;

protected:
    void paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*) override;
    QVariant itemChange(GraphicsItemChange change, const QVariant& value) override;
    void mouseMoveEvent(QGraphicsSceneMouseEvent* event) override;

private:
    void update_polygon();
    void update_points();
    void setChildrenVisible(bool value);
    bool makePolygonClosed();

private:
    PolygonItem* m_item;
    QPolygonF m_polygon;
    bool m_block_on_point_update;
    bool m_close_polygon_request;
};

#endif // BORNAGAIN_GUI_VIEW_SHAPE_POLYGONDISPLAY_H
