//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Shape/PolygonPointDisplay.cpp
//! @brief     Implements class PolygonPointDisplay.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Shape/PolygonPointDisplay.h"
#include "GUI/Model/Mask/MaskItems.h"
#include "GUI/View/ProShape/MaskUtil.h"
#include <QGraphicsSceneMouseEvent>
#include <QPainter>

PolygonPointDisplay::PolygonPointDisplay(PolygonPointItem* item)
    : IShapeDisplay(item)
    , m_item(item)
    , m_on_hover(false)
{
    setFlag(QGraphicsItem::ItemIsMovable);
    setFlag(QGraphicsItem::ItemSendsGeometryChanges);
}

QRectF PolygonPointDisplay::boundingRect() const
{
    return QRectF(-4, -4, 8, 8);
}

MaskRoot* PolygonPointDisplay::parameterizedItem() const
{
    return m_item;
}

void PolygonPointDisplay::updateParameterizedItem(const QPointF& pos)
{
    m_item->setPosX(fromSceneX(pos.x()));
    m_item->setPosY(fromSceneY(pos.y()));
}

void PolygonPointDisplay::update_view()
{
    update();
}

void PolygonPointDisplay::onPropertyChange()
{
    emit propertyChanged();
}

void PolygonPointDisplay::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
{
    painter->setRenderHints(QPainter::Antialiasing);
    QBrush brush = MaskUtil::getSelectionMarkerBrush();
    if (acceptHoverEvents() && m_on_hover)
        brush.setColor(Qt::red);
    painter->setBrush(brush);
    painter->setPen(MaskUtil::getSelectionMarkerPen());
    painter->drawEllipse(boundingRect());
}

void PolygonPointDisplay::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
    updateParameterizedItem(event->scenePos());
}

void PolygonPointDisplay::hoverEnterEvent(QGraphicsSceneHoverEvent* event)
{
    m_on_hover = true;
    emit closePolygonRequest(m_on_hover);
    IShapeDisplay::hoverEnterEvent(event);
}

void PolygonPointDisplay::hoverLeaveEvent(QGraphicsSceneHoverEvent* event)
{
    m_on_hover = false;
    emit closePolygonRequest(m_on_hover);
    IShapeDisplay::hoverLeaveEvent(event);
}
