//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Shape/RectangleBaseDisplay.h
//! @brief     Defines class RectangleBaseDisplay.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEW_SHAPE_RECTANGLEBASEDISPLAY_H
#define BORNAGAIN_GUI_VIEW_SHAPE_RECTANGLEBASEDISPLAY_H

#include "GUI/View/ProShape/SizeHandleElement.h"
#include "GUI/View/Shape/IShapeDisplay.h"
#include <QMap>

//! Base view for all rectangular-like masks.

class RectangleBaseDisplay : public IShapeDisplay {
    Q_OBJECT
public:
    int type() const override { return MaskType::RECTANGLEBASE; }

    explicit RectangleBaseDisplay(MaskRoot* item);

private slots:
    void onSizeHandleElementRequest(bool going_to_resize);

protected:
    void update_view() override;

    virtual void update_position() = 0;
    virtual QRectF mask_rectangle() = 0;
    virtual qreal left() const = 0;
    virtual qreal right() const = 0;
    virtual qreal top() const = 0;
    virtual qreal bottom() const = 0;

    qreal width() const;
    qreal height() const;

    QRectF m_mask_rect; //!< mask rectangle in scene coordinates
    //!< coordinates of corner opposite to the grip corner at the moment it first clicked
    //!< in scene coordinates
    QPointF m_resize_opposite_origin;
    SizeHandleElement* m_activeHandleElement;

private:
    QVariant itemChange(GraphicsItemChange change, const QVariant& value) override;
    void mouseReleaseEvent(QGraphicsSceneMouseEvent* event) override;
    void update_bounding_rect();

    QMap<SizeHandleElement::EHandleLocation, SizeHandleElement*> m_resize_handles;
};

#endif // BORNAGAIN_GUI_VIEW_SHAPE_RECTANGLEBASEDISPLAY_H
